@@ -83,6 +83,22 @@
 //! memory efficient in these cases. There will always be a slight overhead on all
 //! operations on boxed strings, compared to [`String`][String].
 //!
+//! ## Feature Flags
+//!
+//! `smartstring` comes with optional support for the following crates through Cargo
+//! feature flags. You can enable them in your `Cargo.toml` file like this:
+//!
+//! ```no_compile
+//! [dependencies]
+//! smartstring = { version = "*", features = ["proptest", "serde"] }
+//! ```
+//!
+//! | Feature | Description |
+//! | ------- | ----------- |
+//! | [`arbitrary`](https://crates.io/crates/arbitrary) | [`Arbitrary`][Arbitrary] implementation for [`SmartString`][SmartString]. |
+//! | [`proptest`](https://crates.io/crates/proptest) | A strategy for generating [`SmartString`][SmartString]s from a regular expression. |
+//! | [`serde`](https://crates.io/crates/serde) | [`Serialize`][Serialize] and [`Deserialize`][Deserialize] implementations for [`SmartString`][SmartString]. |
+//!
 //! [SmartString]: struct.SmartString.html
 //! [LazyCompact]: struct.LazyCompact.html
 //! [Compact]: struct.Compact.html
@@ -93,6 +109,10 @@
 //! [cmp]: https://doc.rust-lang.org/std/cmp/trait.Ord.html#tymethod.cmp
 //! [transmute]: https://doc.rust-lang.org/std/mem/fn.transmute.html
 //! [tinystr]: https://crates.io/crates/tinystr
+//! [serde]: https://crates.io/crates/serde
+//! [Serialize]: https://docs.rs/serde/latest/serde/trait.Serialize.html
+//! [Deserialize]: https://docs.rs/serde/latest/serde/trait.Deserialize.html
+//! [Arbitrary]: https://docs.rs/arbitrary/latest/arbitrary/trait.Arbitrary.html
 
 #![forbid(rust_2018_idioms)]
 #![deny(nonstandard_style)]
@@ -102,7 +122,7 @@ use std::{
     borrow::{Borrow, BorrowMut},
     cmp::Ordering,
     convert::Infallible,
-    fmt::{Debug, Error, Formatter, Write},
+    fmt::{Debug, Display, Error, Formatter, Write},
     hash::{Hash, Hasher},
     iter::FromIterator,
     mem::{forget, MaybeUninit},
@@ -132,6 +152,15 @@ use casts::{StringCast, StringCastInto, StringCastMut};
 mod iter;
 pub use iter::Drain;
 
+#[cfg(feature = "serde")]
+mod serde;
+
+#[cfg(feature = "arbitrary")]
+mod arbitrary;
+
+#[cfg(feature = "proptest")]
+pub mod proptest;
+
 /// Convenient type aliases.
 pub mod alias {
     use super::*;
@@ -354,10 +383,12 @@ impl<Mode: SmartStringMode> SmartString<Mode> {
             StringCastMut::Boxed(string) => string.string_mut().push(ch),
             StringCastMut::Inline(string) => {
                 let len = string.len();
-                if len + ch.len_utf8() > Mode::MAX_INLINE {
-                    let mut string = self.to_string();
-                    string.push(ch);
-                    self.promote_from(string);
+                let new_len = len + ch.len_utf8();
+                if new_len > Mode::MAX_INLINE {
+                    let mut new_str = String::with_capacity(new_len);
+                    new_str.push_str(string.as_str());
+                    new_str.push(ch);
+                    self.promote_from(new_str);
                 } else {
                     let written = ch.encode_utf8(&mut string.as_mut_slice()[len..]).len();
                     string.set_size(len + written);
@@ -372,10 +403,12 @@ impl<Mode: SmartStringMode> SmartString<Mode> {
         match self.cast_mut() {
             StringCastMut::Boxed(this) => this.string_mut().push_str(string),
             StringCastMut::Inline(this) => {
-                if len + string.len() > Mode::MAX_INLINE {
-                    let mut this = self.to_string();
-                    this.push_str(string);
-                    self.promote_from(this);
+                let new_len = len + string.len();
+                if new_len > Mode::MAX_INLINE {
+                    let mut new_str = String::with_capacity(new_len);
+                    new_str.push_str(this.as_str());
+                    new_str.push_str(string);
+                    self.promote_from(new_str);
                 } else {
                     this.as_mut_slice()[len..len + string.len()].copy_from_slice(string.as_bytes());
                     this.set_size(len + string.len());
@@ -461,10 +494,7 @@ impl<Mode: SmartStringMode> SmartString<Mode> {
     /// If the index doesn't fall on a UTF-8 character boundary, this method panics.
     pub fn remove(&mut self, index: usize) -> char {
         let result = match self.cast_mut() {
-            StringCastMut::Boxed(string) => {
-                let result = string.string_mut().remove(index);
-                result
-            }
+            StringCastMut::Boxed(string) => string.string_mut().remove(index),
             StringCastMut::Inline(string) => {
                 let ch = match string.as_str()[index..].chars().next() {
                     Some(ch) => ch,
@@ -472,9 +502,12 @@ impl<Mode: SmartStringMode> SmartString<Mode> {
                 };
                 let next = index + ch.len_utf8();
                 let len = string.len();
-                unsafe {
-                    (&mut string.as_mut_slice()[index] as *mut u8)
-                        .copy_from(&string.as_slice()[next], len - next);
+                let tail_len = len - next;
+                if tail_len > 0 {
+                    unsafe {
+                        (&mut string.as_mut_slice()[index] as *mut u8)
+                            .copy_from(&string.as_slice()[next], tail_len);
+                    }
                 }
                 string.set_size(len - (next - index));
                 return ch;
@@ -960,6 +993,16 @@ impl<'a, Mode: SmartStringMode> FromIterator<&'a String> for SmartString<Mode> {
     }
 }
 
+impl<Mode: SmartStringMode> FromIterator<char> for SmartString<Mode> {
+    fn from_iter<I: IntoIterator<Item = char>>(iter: I) -> Self {
+        let mut out = Self::new();
+        for ch in iter {
+            out.push(ch);
+        }
+        out
+    }
+}
+
 impl<Mode: SmartStringMode> FromStr for SmartString<Mode> {
     type Err = Infallible;
     fn from_str(s: &str) -> Result<Self, Self::Err> {
@@ -979,12 +1022,6 @@ impl<Mode: SmartStringMode> Into<String> for SmartString<Mode> {
     }
 }
 
-impl<Mode: SmartStringMode> ToString for SmartString<Mode> {
-    fn to_string(&self) -> String {
-        self.as_str().to_string()
-    }
-}
-
 impl<Mode: SmartStringMode> PartialEq<str> for SmartString<Mode> {
     fn eq(&self, other: &str) -> bool {
         self.as_str() == other
@@ -1053,6 +1090,12 @@ impl<Mode: SmartStringMode> Debug for SmartString<Mode> {
     }
 }
 
+impl<Mode: SmartStringMode> Display for SmartString<Mode> {
+    fn fmt(&self, f: &mut Formatter<'_>) -> Result<(), Error> {
+        Display::fmt(self.as_str(), f)
+    }
+}
+
 impl<Mode: SmartStringMode> Write for SmartString<Mode> {
     fn write_str(&mut self, string: &str) -> Result<(), Error> {
         self.push_str(string);
